/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License.
 * See License.txt in the project root for license information.
 */

#include <jni.h>
#include <com_azure_storage_fastpath_driver_FastpathDriver.h>
#include <fpcommon.h>

#include "fake_responses.h"

thread_local FakeResponseContext gFakeResponseContext;

/*
 * Interfaces to be consumed by the FastpathJar endpoint
 */
FakeResponse * LookupResponse(const char *clientRequestID)
{
    FakeResponse *response = gFakeResponseContext.LookupResponse(clientRequestID);
    return response;
}

/*
 * Interfaces to be consumed in Java land
 */

JNIEXPORT void JNICALL Java_com_azure_storage_fastpath_driver_FastpathDriver_registerOpenResponse(
        JNIEnv *env,
        jobject thisObject,
        jstring clRqID,
        jstring jsonRsp)
{
    UNUSED(thisObject);

    gFakeResponseContext.RegisterResponse(env, clRqID, jsonRsp, 0, 0, NULL);
}

JNIEXPORT void JNICALL Java_com_azure_storage_fastpath_driver_FastpathDriver_unregisterOpenResponse(
        JNIEnv *env,
        jobject thisObject,
        jstring clientRequestID,
        jstring jsonResponseString)
{
    UNUSED(thisObject);
    UNUSED(jsonResponseString);

    gFakeResponseContext.UnregisterResponse(env, clientRequestID);
}

JNIEXPORT void JNICALL Java_com_azure_storage_fastpath_driver_FastpathDriver_registerReadResponse(
        JNIEnv *env,
        jobject thisObject,
        jstring clRqID,
        jstring jsonRsp,
        jint rdLen,
        jlong rmtFileOff,
        jobject rmtBuf)
{
    UNUSED(thisObject);

    gFakeResponseContext.RegisterResponse(env, clRqID, jsonRsp, rdLen, rmtFileOff, rmtBuf);
}

JNIEXPORT void JNICALL Java_com_azure_storage_fastpath_driver_FastpathDriver_unregisterReadResponse(
        JNIEnv *env,
        jobject thisObject,
        jstring clientRequestID,
        jstring jsonResponseString)
{
    UNUSED(thisObject);
    UNUSED(jsonResponseString);

    gFakeResponseContext.UnregisterResponse(env, clientRequestID);
}

JNIEXPORT void JNICALL Java_com_azure_storage_fastpath_driver_FastpathDriver_registerCloseResponse(
        JNIEnv *env,
        jobject thisObject,
        jstring clRqID,
        jstring jsonRsp)
{
    UNUSED(thisObject);

    gFakeResponseContext.RegisterResponse(env, clRqID, jsonRsp, 0, 0, NULL);
}

JNIEXPORT void JNICALL Java_com_azure_storage_fastpath_driver_FastpathDriver_unregisterCloseResponse(
        JNIEnv *env,
        jobject thisObject,
        jstring clientRequestID,
        jstring jsonResponseString)
{
    UNUSED(thisObject);
    UNUSED(jsonResponseString);

    gFakeResponseContext.UnregisterResponse(env, clientRequestID);
}

JNIEXPORT void JNICALL Java_com_azure_storage_fastpath_driver_FastpathDriver_registerBadResponseHdr(
        JNIEnv *env,
        jobject thisObject,
        jstring clientRequestID)
{
    UNUSED(thisObject);

    gFakeResponseContext.SetBadResponseHeader(env, clientRequestID);
}
