/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License.
 * See License.txt in the project root for license information.
 */

#ifndef _FAKE_RESPONSES_H_
#define _FAKE_RESPONSES_H_

#include <jni.h>

#include <stdint.h>

#include <mutex>
#include <unordered_map>
#include <string>

class FakeResponse {
public:
    jstring clientRequestIdRef;
    jstring jsonResponseStringRef;
    const char *clientRequestId;
    const char *jsonResponseString;
    uint32_t responseLength;
    uint32_t readLength;
    uint64_t remoteFileOffset;
    int8_t *fakeRemoteBuffer;
    bool badResponseHeader;

    FakeResponse();
    ~FakeResponse();

    int Init(JNIEnv *env,
        jstring fClientRequestId,
        jstring fJsonResponseString,
        jint fReadLength,
        jlong fRemoteFileOffset,
        jobject fRemoteBuffer);
    void CleanupJniRefs(JNIEnv *env);
};

class FakeResponseContext {
private:
    std::mutex lock;
    std::unordered_map<std::string, FakeResponse *> rmap;

public:
    FakeResponseContext();
    ~FakeResponseContext();

    void RegisterResponse(JNIEnv *env,
            jstring fClientRequestId,
            jstring fJsonResponseString,
            jint fReadLength,
            jlong fRemoteFileOffset,
            jobject fRemoteBuffer);
    void UnregisterResponse(JNIEnv *env, jstring fClientRequestId);
    void SetBadResponseHeader(JNIEnv *env, jstring fClientRequestId);
    FakeResponse *LookupResponse(const char *fclientRequestId);
};

FakeResponse *LookupResponse(const char *clReqID);

#endif // _FAKE_RESPONSES_H_
