/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License.
 * See License.txt in the project root for license information.
 */

#include <fpcommon.h>

#include "fastpath.h"

SharedLibContext *gSharedLibContext = NULL;

__attribute__((constructor))
void fastpath_lib_init ()
{
    gSharedLibContext = new SharedLibContext;
}

__attribute__((destructor))
void fastpath_lib_cleanup ()
{
    if (gSharedLibContext != NULL) {
        delete gSharedLibContext;
        gSharedLibContext = NULL;
    }
}

/*
 * Interfaces exposed to our consumers - currently abfs_endpoint.c
 */
int VscOpenBlob (IN const char *clientRequestId,
        IN uint32_t timeoutMsec,
        IN const char *pInputBuf,
        IN uint32_t inputBufLen,
        OUT char *pResponseBuf,
        IN uint32_t responseBufLen,
        OUT int32_t *pLocalErrno,
        OUT uint32_t *pHostResponseLen)
{
    int rc = 0;

    if (pInputBuf == NULL || inputBufLen == 0 || pResponseBuf == NULL ||
            responseBufLen == 0) {
        rc = -1;
        *pLocalErrno = EINVAL;
        goto out;
    }

    rc = gSharedLibContext->vscContext.IssueVscSyncRequest(clientRequestId,
            timeoutMsec,
            pInputBuf,
            inputBufLen,
            NULL,
            0,
            0,
            pResponseBuf,
            responseBufLen,
            pLocalErrno,
            pHostResponseLen);

out:
    return rc;
}

int VscGetBlob(IN const char *clientRequestId,
        IN uint32_t timeoutMsec,
        IN const char *pInputBuf,
        IN uint32_t inputBufLen,
        OUT char *pDataBuf,
        IN uint32_t dataBufLen,
        OUT char *pResponseBuf,
        IN uint32_t responseBufLen,
        OUT int32_t *pLocalErrno,
        OUT uint32_t *pHostResponseLen)
{
    int rc = 0;

    if (pInputBuf == NULL || inputBufLen == 0 || pDataBuf == NULL ||
            dataBufLen == 0 || pResponseBuf == NULL || responseBufLen == 0 ||
            pHostResponseLen == NULL) {
        rc = -1;
        *pLocalErrno = EINVAL;
        goto out;
    }

    rc = gSharedLibContext->vscContext.IssueVscSyncRequest(clientRequestId,
            timeoutMsec,
            pInputBuf,
            inputBufLen,
            pDataBuf,
            dataBufLen,
            0,
            pResponseBuf,
            responseBufLen,
            pLocalErrno,
            pHostResponseLen);

out:
    return rc;
}

int VscCloseBlob(IN const char *clientRequestId,
        IN uint32_t timeoutMsec,
        IN const char *pInputBuf,
        IN uint32_t inputBufLen,
        OUT char *pResponseBuf,
        IN uint32_t responseBufLen,
        OUT int32_t *pLocalErrno,
        OUT uint32_t *pHostResponseLen)
{
    int rc = 0;

    if (pInputBuf == NULL || inputBufLen == 0 || pResponseBuf == NULL ||
            responseBufLen == 0) {
        rc = -1;
        *pLocalErrno = EINVAL;
        goto out;
    }

    rc = gSharedLibContext->vscContext.IssueVscSyncRequest(clientRequestId,
            timeoutMsec,
            pInputBuf,
            inputBufLen,
            NULL,
            0,
            0,
            pResponseBuf,
            responseBufLen,
            pLocalErrno,
            pHostResponseLen);

out:
    return rc;
}
